unit uArcos;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
  Classes,
  SysUtils, xmatdefs, usimplex, uglobs,
  uNodos,
  uActorNodal,
  ucosa,
  uCosaConNombre,
  ufechas, uconstantesSimSEE, uranddispos,
  uFichasLPD;

resourcestring
  rsArcoSimple = 'Arco';

type

  { TFichaArco }

  TFichaArco = class(TFichaLPD)
    rendimiento_: TDAofNReal;
    PagoPorDisponibilidad_USD_MWh: NReal; // [USD/MWh] Pago por Potencia
    peaje_: TDAofNReal;
    PMax_: TDAofNreal;

    fd: NReal; // factor de disponibilidad
    tRepHoras: NReal;

    flg_ConsiderarPeajeEnElDespacho: boolean;
    flg_SumarPeajeAlCDP: boolean;
    factorPeajeCDP: NReal;

    constructor Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
      rendimiento, peaje, PMax: TDAofNReal; fd: NReal; tRepHoras: NReal;
      ConsidearrPeajeEnElDespacho, SumarPeajeAlCDP: boolean;
      FactorPeajeCDP: NReal; PagoPorDisponibilidad_USD_MWh: NReal);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    function InfoAd_: string; override;
    procedure Free; override;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2): TDataColumnListOfCosa; override;
  end;



  { TArco }

  TArco = class(TActorBiNodal)
  public
    NLineasDisponibles: integer;
    PMaxDisponible_: TDAofNReal;
    // resultados del despacho
    P_Entrante: TDAOfNReal; //[MW] Potencia Entrante

    //Costo por poste de la transmisión
    //costo[iposte]:= P[iposte] * pa.peaje * globs.durpos[iposte];
    costo: TDAofNReal;
    // valor por poste del multiplicador de Lagrange de la restrición de PMax
    costo_congestion: TDAOfNReal; //[USD/MW]

    pa: TFichaArco;

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades, lpd: TFichasLPD; Entrada, Salida: TNodo);

    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    procedure PrepararMemoria(globs: TGlobs); override;
    procedure RegistrarParametrosDinamicos; override;
    function InfoAd_: string; override;
    class function DescClase: string; override;

    procedure SorteosDelPaso(sortear: boolean); override;

    procedure opt_nvers(var ivar, ivae, ires: integer); override;
    procedure opt_cargue(s: TSimplex); override;
    procedure opt_fijarRestriccionesDeCaja(s: TSimplex); override;
    procedure opt_leerSolucion(s: TSimplex); override;

    function getNombreVar(ivar: integer; var nombre: string): boolean; override;
    function getNombreRes(ires: integer; var nombre: string): boolean; override;

    procedure dump_Variables(var f: TextFile; charIndentacion: char); override;

    procedure PubliVars; override;
    procedure Free; override;

    function get_pa_FD(kTipoUnidad: integer): NReal; override;
    function get_pa_TMR(kTipoUnidad: integer): NReal; override;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2): TDataColumnListOfCosa; override;

  end;



procedure AlInicio;
procedure AlFinal;

implementation

uses uActores;


//-------------------
// Métodos de TArco
//===================

constructor TArco.Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
  lpdUnidades, lpd: TFichasLPD; Entrada, Salida: TNodo);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades, Entrada, Salida);
  self.lpd := lpd;
end;

constructor TArco.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rd('lpd', TCosa(lpd));
  f.EjecutarLectura;
  lpd.Propietario := self;
end;

procedure TArco.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('lpd', lpd);
end;

procedure TArco.PrepararMemoria(globs: TGlobs);
var
  n, k, j: integer;
  af: TFichaArco;

begin
  inherited prepararMemoria(globs);
  setlength(P_Entrante, globs.NPostes);
  SetLength(costo, globs.NPostes);
  setlength(costo_congestion, globs.NPostes);
  setlength(PMaxDisponible_, globs.NPostes);


  for j := 0 to self.lpd.Count - 1 do
  begin
    af := self.lpd.items[j] as TFichaArco;
    if (length(af.rendimiento_) <> globs.NPostes) then
    begin
      n := length(af.rendimiento_);
      setlength(af.rendimiento_, globs.NPostes);
      setlength(af.peaje_, globs.NPostes);
      setlength(af.PMax_, globs.NPostes);
      for k := n to high(af.rendimiento_) do
        af.rendimiento_[k] := af.rendimiento_[n - 1];
      for k := n to high(af.peaje_) do
        af.peaje_[k] := af.peaje_[n - 1];
      for k := n to high(af.PMax_) do
        af.PMax_[k] := af.PMax_[n - 1];
    end;
  end;

end;

procedure TArco.RegistrarParametrosDinamicos;
begin
  inherited registrarParametrosDinamicos;
  lpd.expandirFichas(globs);
  lpd.RegistrarFichasAActualizar(Self, globs.ActualizadorLPD, @pA, nil);
end;

function TArco.InfoAd_: string;
begin
  Result := '';
end;

class function TArco.DescClase: string;
begin
  Result := rsArcoSimple;
end;


function TArco.get_pa_FD(kTipoUnidad: integer): NReal;
begin
  Result := pa.fd;
end;

function TArco.get_pa_TMR(kTipoUnidad: integer): NReal;
begin
  Result := pa.tRepHoras;
end;

class function TArco.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa;
  xVersion: Integer): TDataColumnListOfCosa;
var
  list:TDataColumnListOfCosa;
begin
  //list:=TDataColumnListOfCosa.Create(TArco, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('lpd', TArco(nil).lpd, TDataTypeDataRowOfCosa, 1, -1, TFichasLPD);
  result:=list;
end;

procedure TArco.SorteosDelPaso(sortear: boolean);
var
  kPoste: integer;

begin
  if globs.ObligarDisponibilidad_1_ then
  begin
    NLineasDisponibles := paUnidades.nUnidades_Instaladas[0]-paUnidades.nUnidades_EnMantenimiento[0];
    for kPoste := 0 to globs.NPostes - 1 do
      PMaxDisponible_[kPoste] := pa.PMax_[kPoste] * NLineasDisponibles;
  end
  else
  if sortear then
  begin
    ActualizarProbabilidadesReparacionYRotura_(pa.fd, pa.tRepHoras);
    NLineasDisponibles := Sorteos_RepRotUnidades;
    for kPoste := 0 to globs.NPostes - 1 do
      PMaxDisponible_[kPoste] := pa.PMax_[kPoste] * NLineasDisponibles;
  end
  else
  begin
    NLineasDisponibles := paUnidades.nUnidades_Instaladas[0]-paUnidades.nUnidades_EnMantenimiento[0];
    for kPoste := 0 to globs.NPostes - 1 do
      PMaxDisponible_[kPoste] := pa.PMax_[kPoste] * pa.fd * NLineasDisponibles;
  end;
end;

procedure TArco.opt_nvers(var ivar, ivae, ires: integer);
begin
  Self.ivar := ivar;
  if NLineasDisponibles > 0 then
    ivar := ivar + globs.NPostes;
end;

procedure TArco.opt_cargue(s: TSimplex);
var
  ibaseResSal, ibaseResEnt: integer;
  iposte: integer;
begin
  if NLineasDisponibles > 0 then
  begin
{$IFDEF SPXCONLOG}
    for iposte := 0 to globs.NPostes - 1 do
      s.set_NombreVar(ivar + iposte, Nombre + '_P[MW]' + IntToStr(iposte + 1));
{$ENDIF}
    // Tenemos que aportar a las restricciones de
    // demanda de los nodos de
    ibaseResEnt := NodoA.ires;
    ibaseResSal := NodoB.ires;
    for iposte := 0 to globs.NPostes - 1 do
    begin
      s.pon_e(ibaseResEnt + iposte, ivar + iposte, -1);
      s.pon_e(ibaseResSal + iposte, ivar + iposte, pa.rendimiento_[iposte]);

      // Ahora agregamos a la función de UTILIDAD (-costo)
      if pa.flg_ConsiderarPeajeEnElDespacho then
        s.pon_e(s.nf, ivar + iposte, -pa.peaje_[iposte] * globs.durpos[iposte]);
    end;
  end;
end;

procedure TArco.opt_fijarRestriccionesDeCaja(s: TSimplex);
var
  iposte: integer;
begin
  if NLineasDisponibles > 0 then
    // Le fijamos como cota máxima PMax a la potencia en todos los postes
    for iposte := 0 to globs.NPostes - 1 do
      s.cota_sup_set(ivar + iposte, PMaxDisponible_[iposte]);
end;

procedure TArco.opt_leerSolucion(s: TSimplex);
var
  iposte: integer;
  m: NReal;
  P: NReal;
begin
  CostoDirectoDelPaso := 0;
  Ingreso_PorDisponibilidad := 0;
  Ingreso_PorEnergia := 0;
  vclear(P_Entrante);
  vclear(P_NodoA);
  vclear(P_NodoB);
  vclear(costo_congestion);

  if NLineasDisponibles > 0 then
  begin
    if pa.PagoPorDisponibilidad_USD_MWh <> 0 then
      for iposte := 0 to globs.NPostes - 1 do
        Ingreso_PorDisponibilidad :=
          Ingreso_PorDisponibilidad + pa.PagoPorDisponibilidad_USD_MWh *
          PMaxDisponible_[iposte] * globs.DurPos[iposte];

    // recuperamos los valores de Potencia despachada
    for iposte := 0 to globs.NPostes - 1 do
    begin
      P := s.xval(ivar + iposte);
      P_Entrante[iposte] := P;
      if pa.flg_SumarPeajeAlCDP then
      begin
        m := P * pa.peaje_[iposte] * globs.DurPos[iposte];
        costo[iposte] := m;
        Ingreso_PorEnergia := Ingreso_PorEnergia + m;
      end;
      costo_congestion[iposte] := -s.xmult(ivar + iposte);
      P_NodoA[iposte] := -P;
      P_NodoB[iposte] := P * pa.rendimiento_[iposte];
    end;

    costoDirectoDelPaso := Ingreso_PorDisponibilidad;
    if pa.flg_SumarPeajeAlCDP then
      costoDirectoDelPaso := costoDirectoDelPaso + Ingreso_PorEnergia *
        pa.factorPeajeCDP;
  end;
end;

function TArco.getNombreVar(ivar: integer; var nombre: string): boolean;
begin
  if (ivar >= self.ivar) and (ivar < self.ivar + globs.NPostes) then
  begin
    nombre := self.Nombre + '_P[MW]' + IntToStr(ivar - self.ivar + 1);
    Result := True;
  end
  else
    Result := False;
end;

function TArco.getNombreRes(ires: integer; var nombre: string): boolean;
begin
  Result := False;
end;

procedure TArco.dump_Variables(var f: TextFile; charIndentacion: char);
var
  iposte: integer;

begin
  inherited dump_Variables(f, charIndentacion);

  Write(f, charIndentacion, 'peaje=');
  for iposte := 0 to high(pa.peaje_) do
  begin
    if iposte > 0 then
      Write(f, ',');
    Write(f, ' ' + FloatToStrF(pa.peaje_[iposte], ffFixed, 10, 3));
  end;
  writeln(f);

  Write(f, charIndentacion, 'PMax=');
  for iposte := 0 to high(pa.PMax_) do
  begin
    if iposte > 0 then
      Write(f, ',');
    Write(f, ' ', FloatToStrF(pa.PMax_[iposte], ffFixed, 10, 3));
  end;
  writeln(f);

  writeln(f);
end;

procedure TArco.PubliVars;
begin
  inherited PubliVars;
  PublicarVariableVR('P', '[MW]', 6, 1, P_Entrante, True, True);
  PublicarVariableVR('Costo', '[USD]', 6, 1, costo, True, True);
  PublicarVariableVR('CostoCongestion', '[USD/MW]', 6, 1, costo_congestion, True, True);
  PublicarVariableNI('NLineasDisponibles', '[u]', NLineasDisponibles, True);
end;

procedure TArco.Free;
begin
  if lpd <> nil then
    lpd.Free;
  setlength(P_Entrante, 0);
  SetLength(costo, 0);
  SetLength(costo_congestion, 0);
  setlength(PMaxDisponible_, 0);
  inherited Free;
end;

//---------------------
//Metodos de TFichaArco
//=====================

constructor TFichaArco.Create(capa: integer; fecha: TFecha;
  periodicidad: TPeriodicidad; rendimiento, peaje, PMax: TDAofNReal;
  fd: NReal; tRepHoras: NReal; ConsidearrPeajeEnElDespacho, SumarPeajeAlCDP: boolean;
  FactorPeajeCDP: NReal; PagoPorDisponibilidad_USD_MWh: NReal);
begin
  inherited Create(capa, fecha, periodicidad);
  self.rendimiento_ := rendimiento;
  self.peaje_ := peaje;
  self.PMax_ := PMax;
  self.fd := fd;
  self.tRepHoras := tRepHoras;
  self.flg_ConsiderarPeajeEnElDespacho := ConsidearrPeajeEnElDespacho;
  self.flg_SumarPeajeAlCDP := SumarPeajeAlCDP;
  self.factorPeajeCDP := FactorPeajeCDP;
  self.PagoPorDisponibilidad_USD_MWh := PagoPorDisponibilidad_USD_MWh;
end;

constructor TFichaArco.Create_ReadFromText(f: TArchiTexto);
var
  rendimiento_aux, peaje_aux, PMax_aux: NReal;
begin
  flg_ConsiderarPeajeEnElDespacho := True;
  flg_SumarPeajeAlCDP := True;
  factorPeajeCDP := 1.0;

  if f.Version < 14 then
  begin
    inherited Create_ReadFromText(f);
    f.IniciarLecturaRetrasada;
    f.rd('rendimiento', rendimiento_aux);
    f.rd('peaje', peaje_aux);
    f.rd('fd', fd);
    f.rd('PMax', PMax_aux);
    f.EjecutarLectura;
    self.tRepHoras := 8;
  end
  else if f.Version < 69 then
  begin
    inherited Create_ReadFromText(f);
    f.IniciarLecturaRetrasada;
    f.rd('rendimiento', rendimiento_aux);
    f.rd('peaje', peaje_aux);
    f.rd('fd', fd);
    f.rd('tRepHoras', tRepHoras);
    f.rd('PMax', PMax_aux);
    f.EjecutarLectura;
  end
  else
  begin
    inherited Create_ReadFromText(f);
    f.IniciarLecturaRetrasada;
    f.rd('rendimiento', rendimiento_);
    f.rd('peaje', peaje_);
    f.rd('PMax', PMax_);
    f.rd('fd', fd);
    f.rd('tRepHoras', tRepHoras);
    if f.Version >= 70 then
    begin
      f.rd('ConsiderarPeajeEnElDespacho', flg_ConsiderarPeajeEnElDespacho);
      f.rd('SumarPeajeAlCDP', flg_SumarPeajeAlCDP);
      f.rd('factorPeajeCDP', factorPeajeCDP);
    end;
    f.EjecutarLectura;
  end;

  if f.Version < 69 then
  begin
    // hago esto y luego en preparar memoria vemos
    // si no conincide la cantidad de postes lo extiendo.
    setlength(rendimiento_, 1);
    setlength(peaje_, 1);
    setlength(PMax_, 1);
    rendimiento_[0] := rendimiento_aux;
    peaje_[0] := peaje_aux;
    PMax_[0] := PMax_aux;
  end;

  if f.Version < 98 then
    PagoPorDisponibilidad_USD_MWh := 0
  else
    f.rd('PagoPorDisponibilidad_USD_MWh', PagoPorDisponibilidad_USD_MWh);

end;

procedure TFichaArco.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('rendimiento', rendimiento_);
  f.wr('peaje', peaje_);
  f.wr('PMax', PMax_);
  f.wr('fd', fd);
  f.wr('tRepHoras', tRepHoras);

  f.wr('ConsiderarPeajeEnElDespacho', flg_ConsiderarPeajeEnElDespacho);
  f.wr('SumarPeajeAlCDP', flg_SumarPeajeAlCDP);
  f.wr('factorPeajeCDP', factorPeajeCDP);

  f.wr('PagoPorDisponibilidad_USD_MWh', PagoPorDisponibilidad_USD_MWh);
end;


function TFichaArco.InfoAd_: string;
begin
  Result := 'PMáx= ' + DAOfNRealToStr_(PMax_, 10, 1, ';') + ' MW, ' +
    'fDisp= ' + FloatToStrF(fd, ffGeneral, 10, 2) + ' p.u., ' +
    'tRep= ' + FloatToStrF(tRepHoras, ffGeneral, 10, 1) + 'h, ' +
    'Rendimiento= ' + DAOfNRealToStr_(rendimiento_, 10, 2, ';') +
    ' p.u., ' + 'peaje= ' + DAOfNRealToStr_(peaje_, 10, 1, ';') + ' USD/MWh';
end;

procedure TFichaArco.Free;
begin
  setlength(rendimiento_, 0);
  setlength(peaje_, 0);
  setlength(PMax_, 0);

  inherited Free;
end;

class function TFichaArco.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa;
  xVersion: Integer): TDataColumnListOfCosa;
var
  list:TDataColumnListOfCosa;
begin
  //list:=TDataColumnListOfCosa.Create(TFichaArco, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('rendimiento', TFichaArco(nil).rendimiento_, TDataTypeDAOfDoubleOfCosa, 1, -1);
  list.AddDataColumn('peaje', TFichaArco(nil).peaje_, TDataTypeDAOfDoubleOfCosa, 1, -1);
  list.AddDataColumn('PMax', TFichaArco(nil).PMax_, TDataTypeDAOfDoubleOfCosa, 1, -1);
  list.AddDataColumn('fd', TFichaArco(nil).fd, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('tRepHoras', TFichaArco(nil).tRepHoras, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('ConsiderarPeajeEnElDespacho', TFichaArco(nil).flg_ConsiderarPeajeEnElDespacho, TDataTypeBooleanOfCosa, 1, -1);
  list.AddDataColumn('SumarPeajeAlCDP', TFichaArco(nil).flg_SumarPeajeAlCDP, TDataTypeBooleanOfCosa, 1, -1);
  list.AddDataColumn('factorPeajeCDP', TFichaArco(nil).factorPeajeCDP, TDataTypeDoubleOfCosa, 1, -1);
  list.AddDataColumn('PagoPorDisponibilidad_USD_MWh', TFichaArco(nil).PagoPorDisponibilidad_USD_MWh, TDataTypeDoubleOfCosa, 1, -1);
  result:=list;
end;

procedure AlInicio;
begin
  registrarClaseDeCosa(TArco.ClassName, TArco);
  registrarClaseDeCosa(TFichaArco.ClassName, TFichaArco);
end;

procedure AlFinal;
begin
end;

end.
