unit uStringGridWithControls;

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, Grids,
  StdCtrls, types, Buttons, ExtCtrls, uvisordetabla;

type

  { TStringGrigPanel }

  TStringGrigPanel=class(TPanel)
    procedure MouseWheelOnPanel (Sender: TObject; Shift: TShiftState;
         WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
  public
    kCol: Integer;
    kRow: Integer;

    constructor Create(AOwner: TComponent; ACol, ARow:Integer);

  end;

  { TSGWC_BitBtn }
  TSGWC_BitBtn = class(TBitBtn)

    procedure MouseWheelOnBitBtn (Sender: TObject; Shift: TShiftState;
         WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);

  public
    iid: integer;
    constructor Create(TheOwner: TComponent; iid: integer; Hint: string='');
    function kRow:Integer;

  end;

  //Tipo de dato de la columna
  TColumnType = (TC_Text, TC_Coltrol);
  TControlType = (CT_CheckBox, CT_ComboBox, CT_BitBtns);

  { TColumnDef }

  TColumnDef=class
    // Tipo de Columna
    ColumnType: TColumnType;

    // Titulo y formato
    Header: string;
    HeaderFont: TFont;

    // Formato del texto de la columna
    ColumnFont: TFont;

    // Tipo de control de la columna
    ControlType: TControlType;

    // Identificador de BitBtns
    iids: array of integer;

    constructor CreateTextColumn (xHeader: string; xHeaderFont: TFont=nil; xColFont: TFont=nil);

    constructor CreateBitBtnColumn (x_iids: array of Integer; xHeader: string;
         xHeaderFont: TFont=nil);

    constructor CreateCheckBoxColumn (xHeader: string; xHeaderFont: TFont=nil;
      x_hint: string='' );

    constructor CreateComboBoxColumn (xHeader: string; xHeaderFont: TFont=nil;
      x_hint:string='' );

  end;

  // El indice en el array se correcponde con la columna
  TColumnsDef = array of TColumnDef;

  { TStringGridWithControls }

  TStringGridWithControls = class(TStringGrid)
  protected

    _ImageList: TImageList;
    _maxTopRow:Integer;
    _maxHeight: Integer;
    _maxWidth:Integer;

    ColumnsDefinition: TColumnsDef;

    procedure TopLeftChanged(Sender: TObject);
    procedure MouseWheelOnSGWC (Sender: TObject; Shift: TShiftState;
         WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);

    function GetControlOnCell(kCol, kRow: integer): TStringGrigPanel;
    procedure SetControlOnCell(kCol, kRow: integer; AValue: TStringGrigPanel);

    property ControlOnCell[kCol, kRow: integer]: TStringGrigPanel
      read GetControlOnCell write SetControlOnCell;
    procedure MoveControl(AControl: TControl; NewPosition: TPoint;
      isVisible: boolean = True);

    procedure BitBtn_OnClick( Sender: TObject); virtual;
    procedure CheckBox_OnClick( Sender: TObject); virtual;

    procedure Borrar_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure BorrarDisabled_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure Editar_onClick( nidRec: string; kFila: integer ); virtual;
    procedure Ver_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure VerNaranja_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure Bajar_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure BajarDelTodo_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure Configurar_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure Email_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure Info_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure EmailDisabled_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure SemaforoRojo_OnClick( nidRec: string; kFila: integer); virtual;
    procedure SemaforoVerde_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure SubirDelTodo_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure Subir_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure Lupa_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure checkbox_0_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure checkbox_1_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure clonar_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure fdownload_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure fupload_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure radiobutton_0_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure radiobutton_1_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure reset_OnClick( nidRec: string; kFila: integer ); virtual;

    // Escribe el texto en la celda.
    procedure wrText(kCol, kRow: Integer; Text: string);

    // Inserta TCheckBox en la celda kCol, kRow
    procedure wrCheckBox(kCol, kRow: Integer; Hint: string = '');

    // Inserta TComboBox en la celda kCol, kRow
    procedure wrComboBox(kCol, kRow: Integer; Hint: string = '');

    // Escribe un array de BitBtn en la misma celda
    procedure wrBitBtns(kCol, kRow: integer; iids: array of Integer);

    procedure wrControl(kCol, kRow: Integer);

    procedure AutoSizeColumns;
    procedure AutoSizeColumn(aCol: Integer);
    procedure ReArrangeControls;

    procedure SetDimension(nFixedRows, nRows, nFixedCols, nCols: Integer);

    // Crea el encabezado de la tabla
    procedure SetHeader;

  public
    { public declarations }

    constructor Create(AOwner: TComponent; AName: string);
    procedure CreateHeader();virtual;abstract;

    procedure Refresh();virtual;abstract;
    procedure Clear();virtual;abstract;

    //Inserta una fila nueva con los valores y los controles (según corresponda)
    procedure InsertRow(Values: TStrings);virtual;

    procedure DeleteRow(kRow:Integer);override;

    procedure ClearAll();virtual;
    procedure Free;

  end;

implementation

{ TColumnDef }

constructor TColumnDef.CreateTextColumn(xHeader: string; xHeaderFont: TFont;
  xColFont: TFont);
begin
  self.ColumnType:=TC_Text;
  self.Header:=UpperCase(xHeader);
end;

constructor TColumnDef.CreateBitBtnColumn(x_iids: array of Integer;
  xHeader: string; xHeaderFont: TFont);
var
  i: Integer;
begin
  self.ColumnType:=TC_Coltrol;
  self.ControlType:=CT_BitBtns;
  self.Header:=UpperCase(xHeader);

  SetLength(self.iids, Length(x_iids));
  for i:=0 to Length(x_iids)-1 do
    self.iids[i]:=x_iids[i];

end;

constructor TColumnDef.CreateCheckBoxColumn(xHeader: string;
  xHeaderFont: TFont; x_hint: string);
begin

end;

constructor TColumnDef.CreateComboBoxColumn(xHeader: string;
  xHeaderFont: TFont; x_hint: string);
begin
    self.Header:=UpperCase(xHeader);

end;

{ TStringGrigPanel }

procedure TStringGrigPanel.MouseWheelOnPanel(Sender: TObject;
  Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint;
  var Handled: Boolean);
var
  sg:TStringGridWithControls;
begin
  sg:=TStringGridWithControls(TStringGrigPanel(sender).Owner);
  sg.MouseWheelOnSGWC(Sender, Shift, WheelDelta, MousePos, Handled);
end;

constructor TStringGrigPanel.Create(AOwner: TComponent; ACol,
  ARow: Integer);
begin
  inherited Create(AOwner);
  self.kCol:=ACol;
  self.kRow:=ARow;
  self.OnMouseWheel:=MouseWheelOnPanel;

end;

{ TSGWC_BitBtn }

procedure TSGWC_BitBtn.MouseWheelOnBitBtn(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
var
  sg:TStringGridWithControls;
begin
  sg:=TStringGridWithControls(TSGWC_BitBtn(sender).Owner);
  sg.MouseWheelOnSGWC(Sender, Shift, WheelDelta, MousePos, Handled);

end;

constructor TSGWC_BitBtn.Create(TheOwner: TComponent; iid: integer; Hint: string
  );
begin

  Inherited Create(TheOwner);
  self.iid:=iid;
  if Hint<>'' then
    self.Hint:= Hint
  else
    self.Hint:=hintPorDefecto(iid);

  self.Caption:='';
  self.OnMouseWheel:=MouseWheelOnBitBtn;

end;

function TSGWC_BitBtn.kRow: Integer;
begin

end;

{ TStringGridWithControls }

procedure TStringGridWithControls.TopLeftChanged(Sender: TObject);
begin
  reArrangeControls;
end;

function TStringGridWithControls.GetControlOnCell(kCol, kRow: integer): TStringGrigPanel;
begin
  Result:=nil;
  if Objects[kCol, kRow] is TStringGrigPanel then
    Result :=TStringGrigPanel (Objects[kCol, kRow]);
end;

procedure TStringGridWithControls.SetControlOnCell(kCol, kRow: integer;
  AValue: TStringGrigPanel);
begin
  Objects[kCol, kRow] := AValue;
end;

procedure TStringGridWithControls.MoveControl(AControl: TControl;
  NewPosition: TPoint; isVisible: boolean);
var
  Width, Height: integer;
begin
  AControl.Visible := isVisible;
  Width := AControl.Width;
  Height := AControl.Height;
  AControl.SetBounds(NewPosition.x, NewPosition.y, Width, Height);
end;

procedure TStringGridWithControls.BitBtn_OnClick( Sender: TObject);
var
  btn:TSGWC_BitBtn;
  nidRec: String;
  tipoBoton: integer;
  kFila: integer;
begin
  btn:= TSGWC_BitBtn(Sender);
  kFila:=TStringGrigPanel(btn.Parent).kRow;
  tipoBoton:= btn.iid;
  case tipoBoton of
    iid_Borrar: Borrar_OnClick( nidRec, kFila );
    iid_BorrarDisabled: BorrarDisabled_OnClick( nidRec, kFila );
    iid_Editar: Editar_OnClick( nidRec, kFila );
    iid_Ver: Ver_OnClick( nidRec, kFila );
    iid_VerNaranja: VerNaranja_OnClick( nidRec, kFila );
    iid_Bajar: Bajar_OnClick( nidRec, kFila );
    iid_BajarDelTodo: BajarDelTodo_OnClick( nidRec, kFila );
    iid_Configurar: Configurar_OnClick( nidRec, kFila );
    iid_Email: Email_OnClick( nidRec, kFila );
    iid_Info: Info_OnClick( nidRec, kFila );
    iid_EmailDisabled: EmailDisabled_OnClick( nidRec, kFila );
    iid_SemaforoRojo: SemaforoRojo_OnClick( nidRec, kFila );
    iid_SemaforoVerde: SemaforoVerde_OnClick( nidRec, kFila );
    iid_SubirDelTodo: SubirDelTodo_OnClick( nidRec, kFila );
    iid_Subir: Subir_OnClick( nidRec, kFila );
    iid_Lupa: Lupa_OnClick( nidRec, kFila );
    iid_checkbox_0: checkbox_0_OnClick( nidRec, kFila );
    iid_checkbox_1: checkbox_1_OnClick( nidRec, kFila );
    iid_clonar: clonar_OnClick( nidRec, kFila );
    iid_fdownload: fdownload_OnClick( nidRec, kFila );
    iid_fupload: fupload_OnClick( nidRec, kFila );
    iid_radiobutton_0: radiobutton_0_OnClick( nidRec, kFila );
    iid_radiobutton_1: radiobutton_1_OnClick( nidRec, kFila );
    iid_Reset: reset_OnClick( nidRec, kFila );
  end;
end;

procedure TStringGridWithControls.CheckBox_OnClick( Sender: TObject);
begin
  showMessage( 'Click on: '+ TComponent( Sender ).Name );
end;

procedure TStringGridWithControls.Borrar_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'Borrar_OnClick Sin escribir..' + nidRec );
  DeleteRow(kFila);
end;

procedure TStringGridWithControls.BorrarDisabled_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'BorrarDisabled_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.Editar_onClick( nidRec: string; kFila: integer );
begin
  showmessage( 'Editar_onClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.Ver_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'Ver_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.VerNaranja_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'VerNaranja_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.Bajar_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'Bajar_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.BajarDelTodo_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'BajarDelTodo_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.Configurar_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'Configurar_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.Email_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'Email_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.Info_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'Info_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.EmailDisabled_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'EmailDisabled_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.SemaforoRojo_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'SemaforoRojo_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.SemaforoVerde_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'SemaforoVerde_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.SubirDelTodo_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'SubirDelTodo_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.Subir_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'Subir_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.Lupa_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'Lupa_OnClick..' + nidRec );
end;
procedure TStringGridWithControls.checkbox_0_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'checkbox_0_OnClick..' + nidRec );
end;
procedure TStringGridWithControls.checkbox_1_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'checkbox_1_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.clonar_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'clonar_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.fdownload_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'fdownload_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.fupload_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'fupload_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.radiobutton_0_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'radiobutton_0_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.radiobutton_1_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'radiobutton_1_OnClick Sin escribir.. ' + nidRec );
end;
procedure TStringGridWithControls.reset_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'reset_OnClick Sin escribir.. ' + nidRec );
end;


procedure TStringGridWithControls.ReArrangeControls;
var
  kCol, kRow: integer;
  AControl: TStringGrigPanel;
begin

  _maxTopRow:=Self.RowCount-Self.VisibleRowCount;
  for kCol := 0 to ColCount - 1 do
    for kRow := 0 to RowCount - 1 do
    begin
      AControl := ControlOnCell[kCol, kRow];
      if Assigned(AControl) and (AControl is TStringGrigPanel) then
        begin
          MoveControl(AControl, CellRect(kCol, kRow).TopLeft,
            ((LeftCol <= kCol) and (TopRow <= kRow)));

          AControl.kCol:=kCol;
          AControl.kRow:=kRow;

        end;
    end;
end;

procedure TStringGridWithControls.Free;
var
  kCol, kRow: integer;
  AControl: TControl;
begin
  for kCol := 0 to ColCount - 1 do
    for kRow := 0 to RowCount - 1 do
    begin
      AControl := ControlOnCell[kCol, kRow];
      if AControl <> nil then
        AControl.Free;
    end;
  inherited Free;
end;

constructor TStringGridWithControls.Create(AOwner: TComponent; AName: string);
begin

  inherited Create(AOwner);

  self.Name := AName;
  self.ScrollBars := ssBoth;
  self.DefaultRowHeight:=25;

  self.Options := [goFixedVertLine, goFixedHorzLine, goVertLine, goHorzLine];

  self.TopLeftChanged(self);
  self.OnTopLeftChanged := self.TopLeftChanged;
  self.OnMouseWheel:=self.MouseWheelOnSGWC;

  _maxTopRow:=0;
  self._ImageList:=DataModule2.ImageList1;

  SetDimension(0, 0, 0, 0);

  SetLength(ColumnsDefinition, 0);
end;

procedure TStringGridWithControls.InsertRow(Values: TStrings);
var
  nCols: Integer;
  i: Integer;
  kRow, kCol: Integer;
begin

  nCols:=Length(ColumnsDefinition);
  i:=0;

  RowCount:=RowCount+1;
  kRow:=RowCount-1;

  for kCol:=0 to nCols-1 do
    if ColumnsDefinition[i].ColumnType=TC_Text then
    begin
      wrText(i, kRow, Values[i]);
      i:=i+1;
    end
    else
    begin
      wrControl(i, kRow);
    end;

end;

procedure TStringGridWithControls.SetDimension(nFixedRows, nRows, nFixedCols,
  nCols: Integer);
begin

  self.RowCount:=nRows;
  self.FixedRows:=nFixedRows;

  self.ColCount:=nCols;
  self.FixedCols:=nFixedCols;

end;

procedure TStringGridWithControls.SetHeader;
var
  nCols: Integer;
  i: Integer;
  gct: TGridColumnTitle;
  gc: TGridColumn;
  ntc: Integer;
begin

  ClearAll();

  self.CreateHeader();

  if Length(ColumnsDefinition)=0 then
    Exit;

  nCols:=Length(ColumnsDefinition);

  // Agraga las columnas y la primera fila para el encabezado
  self.RowCount:=1;
  self.FixedRows:=1;

  for i:=0 to nCols-1 do
  begin
    gc:=self.Columns.Add;
    gc.Title.Caption:=ColumnsDefinition[i].header;
  end;
end;

procedure TStringGridWithControls.MouseWheelOnSGWC(Sender: TObject;
  Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint;
  var Handled: Boolean);
begin
  if WheelDelta < 0 then
  begin
    if TopRow < _maxTopRow then
      TopRow:=TopRow+1;
  end
  else
  begin
    if TopRow>1 then
      TopRow:=TopRow-1;
  end;
end;

procedure TStringGridWithControls.wrText(kCol, kRow: Integer; Text: string);
begin
  self.Cells[kCol, kRow] := Text;
end;

procedure TStringGridWithControls.wrCheckBox(kCol, kRow: Integer; Hint: string);
var
  R: Classes.TRect;
  P: TStringGrigPanel;
  cb: TCheckBox;
  i, j: Integer;
begin

  R := self.CellRect(kCol, kRow);
  P:=TStringGrigPanel.Create(self, kCol, kRow);
  P.BoundsRect:=R;
  P.Parent:=self;
  P.Color:=clWhite;

  P.Width:=(R.Bottom - R.Top)+2;

  cb := TCheckBox.Create(self);
  cb.Parent := P;
  cb.Caption:='';
  cb.Hint:=Hint;
  i:=trunc(P.Width/2);
  j:=trunc(P.Height/2);

  cb.Left := i - trunc(cb.Width/2)+3;
  cb.Top := j - trunc(cb.Height/2);

  ControlOnCell[kCol, kRow] := P;

end;

procedure TStringGridWithControls.wrComboBox(kCol, kRow: Integer; Hint: string);
begin
  ShowMessage('TStringGridWithControls.wrComboBox: No implementado aún.');
end;

procedure TStringGridWithControls.wrBitBtns(kCol, kRow: integer;
  iids: array of Integer);
var
  R: Classes.TRect;
  P: TStringGrigPanel;
  Btn: TSGWC_BitBtn;
  i: Integer;
  nBtns:Integer;
  //a: TActionButtonKind;

begin

  R := self.CellRect(kCol, kRow);
  P:=TStringGrigPanel.Create(self, kCol, kRow);
  P.BoundsRect:=R;
  P.Parent:=self;
  P.Color:=clBlack;

  nBtns:=0;

  //for a:=High(TActionButtonKind) downto Low(TActionButtonKind) do
    //if a in iids then

    nBtns:=Length(iids);

    //for a in iids do
    for i:=Length(iids)-1 downto 0 do
    begin


      Btn := TSGWC_BitBtn.Create(self, iids[i]);

      Btn.Height:=R.Bottom - R.Top;
      Btn.Width:=R.Bottom - R.Top;
      Btn.Parent:= P;
      Btn.OnClick:= BitBtn_OnClick;
      Btn.Align:=alLeft;
      if Assigned(_ImageList) then
        _ImageList.GetBitmap( iids[i],  Btn.Glyph );

      //Btn.Spacing := 1;
      Btn.Margin:=0;
    end;


  P.Width:=nBtns*(R.Bottom - R.Top)+2;

  ControlOnCell[kCol, kRow] := P;

end;

procedure TStringGridWithControls.wrControl(kCol, kRow: Integer);
var
  ColDef: TColumnDef;
  i: Integer;
begin
  ColDef:=ColumnsDefinition[kCol];
  case ColDef.controlType of
     CT_BitBtns:
        wrBitBtns(kCol, kRow, ColDef.iids);

     CT_CheckBox:
        wrCheckBox(kCol, kRow);

     CT_ComboBox:
        wrComboBox(kCol, kRow);

  end;
end;

procedure TStringGridWithControls.DeleteRow(kRow: Integer);
var
  c: TControl;
  i: Integer;
begin
  for i:=self.ColCount-1 downto 0 do
  begin
    c:=ControlOnCell[i,kRow];
    if Assigned(c) then
      c.Free;
  end;
  inherited DeleteRow(kRow);

  ReArrangeControls;
end;

procedure TStringGridWithControls.ClearAll;
var
  i: Integer;
begin
  self.FixedRows:=0;
  self.FixedCols:=0;

  for i:=RowCount-1 downto 0 do
    DeleteRow(i);

  inherited Clear;
end;

procedure TStringGridWithControls.AutoSizeColumns;
var
  i: Integer;
begin
  for i:=0 to ColCount-1 do
    if ColumnsDefinition[i].ColumnType = TC_Text then
      inherited AutoAdjustColumn(i)
    else
      AutoSizeColumn(i);

  ReArrangeControls;
end;

procedure TStringGridWithControls.AutoSizeColumn(aCol: Integer);
var
  maxHeight, maxWidth:integer;
  iRow: Integer;
  ctrl: TControl;
  theCol: TGridColumn;
begin
  maxHeight:=0;
  maxWidth:=0;
  for iRow:=0 to self.RowCount-1 do
  begin
    ctrl:=ControlOnCell[aCol, iRow];
    if Assigned(ctrl) then
    begin
      if ctrl.Width>maxWidth then
        maxWidth:=ctrl.Width;
      if ctrl.Height>maxHeight then
        maxHeight:=ctrl.Height;
    end;
  end;

  theCol := ColumnFromGridColumn(aCol);
  theCol.Width:=maxWidth+1;
end;


end.
